# = About lib/tarpaulin.rb
#
# By requiring <tt>lib/tarpaulin</tt>, you can load Tarpaulin's dependencies
# (the lazy library, the flash hash, and whatever else)
# (,)
# as well as the full set of Tarpaulin classes.
#
# For a full list of features and instructions, see the README.


# documenfuckingtation?
require 'rubygems'
require "bundler/setup"

# http://grosser.it/2009/07/01/getting-the-caller-method-in-ruby/
# this is like parse_caller() in ActionMailer apparently
module Context
  
  # return [n, dirname, filename or context, line num, self, function]
  def self.determine(skip=1, &block)
    begin
      # cheat, just in case it is nil, rather than handling that special case for now
      # cuz it's always possible to call a func that takes a block without an actual block :(
      # i suppose i could raise an ExpectingBlock Error
      b = block.send(:binding)
      snarf = caller(0)[skip] # defaults to 1
      # [0] is us --- ["../y_ruby/c.rb:9:in `determine'",
      #err "caller(): #{snarf.inspect}"
      matched = /^(.+?)[:](\d+)(.*)$/.match(snarf) # +? non-greedy
      #err "matched /^(.+?)[:](\d+)(.*)$/: #{matched}"
      # the whole thing is matched[0]
      context = matched[1] # is file or (eval) or (irb) or ...
      line_no = matched[2].to_i
      three = matched[3]
      #err "context = matched[1]: #{context}"
      #err "line_no = matched[2]: #{line_no}"
      #err "three = matched[2]: #{three}"
      #err "three is empty?: #{three.length.zero?.inspect}"
      fn = nil
      unless three.length.zero?
        matched = /^:in `(.+?)'$/.match(three)
        #err "matched /^:in `(.+?)'$/: #{matched}"
        fn = matched[1]
        #err "fn = matched[1]: #{fn}"
      end
      first_letter = context[0]
      #err "first_letter = context[0]: #{first_letter}"
      case first_letter
        when 40 # _(_ we do not handle (irb) and (eval) and whatever else yet
          {:type => 0, :path => Dir.getwd, :location => context, :line_number => line_no, :self => eval("self", b), :function => fn}
          #return Dir.getwd + '/' + string
        when 47 # _/_ absolute
          {:type => 1, :path => File.dirname(context), :location => File.basename(context), :line_number => line_no, :self => eval("self", b), :function => fn}
          #return File.dirname(context) + '/' + string
        when 46 # _._ relative ... to what?! i suppose the working dir unless it was changed :( (need a per binding ENV?)
          {:type => 2, :path => File.expand_path(File.dirname(context), ENV['PWD']), :location => File.basename(context), :line_number => line_no, :self => eval("self", b), :function => fn}
          #return File.expand_path(File.dirname(context)) + '/' + string
        else # bare file name i guess unless something weird is happening
          {:type => 3, :path => File.expand_path(File.dirname(context), ENV['PWD']), :location => File.basename(context), :line_number => line_no, :self => eval("self", b), :function => fn}
          #raise NameError.new("odd name found in backtrace -- " + context)
      end # case
    end # begin
  end # determine()
  
end # module Context

# Recognizing that Dir.getwd is not always the dir of the file being interpreted
#
# Thus the pattern has been 
# $:.unshift(File.dirname(__FILE__))
# which according to http://gilesbowkett.blogspot.com/2009/04/unshiftfiledirnamefile.html
# This shit is evil. People are undecided as to how evil, but evil it is.
#
# Preferable is 
# require File.expand_path(File.dirname(__FILE__)) + "foo"
#
# but then you have a load of File.expand_path(File.dirname(__FILE__)) everywhere
# so it would be nice to have File.here , as in
# require File.here "foo"
class File
  
  def File.here (string)
    return Context::determine(2){}[:path] + '/' + string
  end

end

# Tarpaulin is a module containing all of the great Tarpaulin classes that
# do such an excellent job. These are snippets of code common to my Camping apps
#
# * custom class Logger
#
# * catch all class Endless
#
# * Tarpaulin main module and helpers and include modules
#
# * Tilt template for ClearSilver
#
module Tarpaulin
  version_file = File.expand_path(File.here "/../VERSION")
  VERSION      = IO.read(version_file).strip
end

require File.here "tarpaulin/logger"
require File.here "tarpaulin/endless"
require File.here "tarpaulin/tilt/clearsilver_template"
require File.here "tarpaulin/camping"
require File.here "tarpaulin/camping/flash"
require File.here "tarpaulin/camping/filter"
require File.here "tarpaulin/camping/accept_language"
