#
# included into top-level TheApp module
#
# overrides r404, defines r403
# tells Tarpaulin which Controller in TheApp handles static files
# 
# don't need to require anything as these are modules that are
# included elsewhere and those elsewhere require stuff :)
#
# A. use #included and then call TheApp for stuff we need (like #link_controller)
# B. what is the difference between #R, #R, #D, #URL ???
# C. use decent error pages, 403, 404, and Ruby Inspect (html version) in dev mode
# D. do link_ methods need to use #R or some variant?
module Tarpaulin
  
  def self.included(mod)
    $DEBUG and $stderr.puts "mod is #{@mod}"
    @mod = mod
    $DEBUG and $stderr.puts "now it is #{@mod}"
  end
  
  # check for Context, only show a.inspect in dev mode
  def r403(f=nil, &block)
    #$stderr.puts "in 403"
    h = {'Content-Type' => 'text/html; charset=utf-8'}
    a = eval("Context::determine{}", block)
    if f.nil?
      f = block.call
    end
    extra = (ENV['MODE'] == 'development') ? "<br><hr>#{a.inspect}" : ''
    html_msg = "<!DOCTYPE html><html lang='en'><head>"+
               "<title>&#171; Uh oh &#187;</title><body><span class='error'>"+
               "Site error (BEEP!) 403: Invalid Path: #{f}#{extra}</span></body>"
    r(403, html_msg, h)
  end

  def r404(f=nil, &block) # compat with Camping
    #$stderr.puts "in 404"
    h = {'Content-Type' => 'text/html; charset=utf-8'}
    a = eval("Context::determine{}", block)
    if f.nil?
      f = block.call
    end
    extra = (ENV['MODE'] == 'development') ? "<br><hr>#{a.inspect}" : ''
    html_msg = "<!DOCTYPE html><html lang='en'><head>"+
               "<title>&#171; Uh oh &#187;</title><body><span class='error'>"+
               "Site error (BEEP!) 404: File not found: #{f}#{extra}</span></body>"
    r(404, html_msg, h)
  end
  
  # included into TheApp::Helpers module

  module Helpers
    
    def self.included(mod)
      $DEBUG and $stderr.puts "h:mod is #{@mod}"
      @mod = mod
      $DEBUG and $stderr.puts "now it is #{@mod}"
    end
    
    # should be configified
    def document_root # or whatever ...
      '/var/www/localhost/htdocs/'
      # /Users/jollyrotten/Code/poetify/y_ruby
      # File.dirname(__FILE__)
    end
      
    # should be configified
    def http_host # or whatever ...
      @env['HTTP_HOST']
    end
      
    # could do all these with a bit of meta-programming
    
    IMAGE = 'images'
    def image_dir
      File.join(document_root, IMAGE)
    end
    
    CSS = 'cascading_stylesheets' # cascading_stylesheets
    def css_dir
      File.join(document_root, CSS)
    end
    
    JS = 'javascripts'
    def js_dir
      File.join(document_root, JS)
    end
    
    def prepare_root
      if @root.length > 0 and @root[0] == '/'.ord
        root = @root.reverse.chop!.reverse
      else
        root = @root
      end
      root
    end
    
    def stylesheet_link_tag(stylish)
      type = "text/css"
      href = if stylish.start_with?("http") or stylish.start_with?("//")
        "#{stylish}"
      else
        root = prepare_root
        (root and root.length > 0) ?
          R(link_controller, root, Tarpaulin::Helpers::CSS, "#{stylish}.css") :
          R(link_controller, Tarpaulin::Helpers::CSS, "#{stylish}.css")
      end
      
      Markaby::Builder.new.capture {
        link :rel => "stylesheet",
             :type => type,
             :media => "screen",
             :href => href
      }
    end
    
    def javascript_link_tag(scriptish)
      type = "text/javascript"
      src = if scriptish.start_with?("http") or scriptish.start_with?("//")  # 3rd party css? really?
        "#{scriptish}"
      else
        root = prepare_root
        (root and root.length > 0) ?
          R(link_controller, root, Tarpaulin::Helpers::JS, "#{scriptish}.js") :
          R(link_controller, Tarpaulin::Helpers::JS, "#{scriptish}.js")
      end

      Markaby::Builder.new.capture {
        script(:type => type, :src => src) do
        end
      }
    end
    
    @@empty_hdf = Neo::Hdf.new
    
    def hdf_render *a
      block_given? ?
         render(*a, &Proc.new) :
         render(*a) { @@empty_hdf }
    end

  end # module Helpers

end # module Tarpaulin
